<template>
  <div class="form-container">

    <van-sticky>
      <van-nav-bar
        :title="routerQuery.title || '案件处置处理'"
        left-arrow
        @click-left="_ => $router.go(-1)"
      />
    </van-sticky>

    <van-form
      scroll-to-error
      label-width="70px"
      @submit="onSubmit"
      @failed="onFailed"
      ref="vanForm"
    >
      <van-field
        name="walkTime"
        label="巡逻时间"
        input-align="right"
        v-model="formData.walkTime"
        placeholder="请选择巡逻时间"
        required
        :rules="[{required: true}]"
        readonly
        clickable
        @click="walkTimeCalendar = true"
      />
      <van-field
        name="caseType"
        label="案件类型"
        input-align="right"
        v-model="formData.caseType"
        placeholder="请选择案件类型"
        required
        :rules="[{required: true}]"
        readonly
        clickable
        @click="caseTypePicker = true"
      />
      <van-field
        name="caseSrc"
        label="案件来源"
        input-align="right"
        v-model="formData.caseSrc"
        placeholder="请选择案件来源"
        required
        :rules="[{required: true}]"
        readonly
        clickable
        @click="caseSrcPicker = true"
      />
      <van-field
        name="caseReason"
        label="案由"
        input-align="right"
        rows="2"
        autosize
        placeholder="请输入案由"
        type="textarea"
        required
        :rules="[{required: true}]"
        v-model="formData.caseReason"
      />
      <van-field
        name="walkAddr"
        label="巡逻地址"
        input-align="right"
        v-model="formData.walkAddr"
        rows="2"
        autosize
        type="textarea"
        placeholder="请输入巡逻地址"
      />
      <van-field
        name="memo"
        label="备注"
        label-width="100px"
        input-align="right"
        v-model="formData.memo"
        rows="2"
        autosize
        type="textarea"
        placeholder="请输入备注"
      />
      <van-field label="巡逻附件图片" label-width="100px" name="caseFileArr">
        <template #input>
          <van-uploader
            v-model="formData.caseFileArr"
            :before-read="beforeRead"
            :after-read="afterRead"
            :before-delete="beforeDelete"
          />
        </template>
      </van-field>

      <div class="footer">
        <van-button
          :loading="loading"
          plain
          native-type="button"
          @click="reset(true)"
        >
          重置
        </van-button>
        <van-popover
          placement="top"
          v-model="showPopover"
          trigger="click"
          :actions="actions"
          @select="popoverSelect"
          @open="popoverOpen"
        >
          <template v-slot:reference>
            <van-button
              :loading="loading"
              native-type="button"
              type="info">
              案件录入
            </van-button>
          </template>
        </van-popover>
      </div>
    </van-form>

    <!--  选择器  -->
    <template>
      <!--   案件类型   -->
      <van-popup v-model="caseTypePicker" position="bottom">
        <van-picker
          title="案件类型"
          show-toolbar
          value-key="name"
          ref="caseType"
          :columns="caseTypeOpt"
          @confirm="
            ({name}) => {
              formData.caseType = name
              caseTypePicker = false
            }
          "
          @cancel="caseTypePicker = false"
        />
      </van-popup>

      <!--   案件来源   -->
      <van-popup v-model="caseSrcPicker" position="bottom">
        <van-picker
          title="案件来源"
          show-toolbar
          value-key="name"
          ref="caseSrc"
          :columns="caseSrcOpt"
          @confirm="
            ({name}) => {
              formData.caseSrc = name
              caseSrcPicker = false
            }
          "
          @cancel="caseSrcPicker = false"
        />
      </van-popup>

      <!--   巡逻时间   -->
      <van-calendar
        color="#4187f2"
        v-model="walkTimeCalendar"
        :min-date="new Date(2010, 0, 1)"
        :max-date="new Date()"
        @confirm="walkTimeCalendarConfirm"
        title="巡逻时间选择"
      />
    </template>

    <!--功能弹出层-->
    <template>
      <!--   案件处置处理表单弹出层   -->
      <DispositionPopup
        :isShow.sync="dispositionPopup"
        :loading.sync="loading"
        :simpleHandleTypeOpt="simpleHandleTypeOpt"
        @onSubmit="popupFormSubmit"
        ref="dispositionPopup"
      />

      <!--   上报部门选择   -->
      <van-action-sheet v-model="reportingPopup" title="请选择案件上报部门" style="height: 50vh;"
                        cancel-text="确定"
                        @cancel="onCancel">
        <sectionTree ref="secondSectionTree" @handleOrg="handleOrg" />
      </van-action-sheet>
    </template>

    <van-overlay :show="dataLoading">
      <van-loading color="#1989fa" size="24px">案件信息加载中...</van-loading>
    </van-overlay>
  </div>
</template>
<script>
import {edit, fileUpload, details} from '@/api/Mobile/caseHandling'
import dictUtils from '@/components/dict/DictSelectUtils'
import {dictionaryEscape} from '@/utils/dictionary'
import _ from 'lodash'
import {Toast, Dialog} from 'vant'
import DispositionPopup from './components/DispositionPopup'
import sectionTree from '@/components/sectionTree/index.vue'
import moment from 'moment'

export default {
  name: 'DraftDetails',
  components: {sectionTree, DispositionPopup},
  data() {
    return {
      formData: {
        caseFile: [],
        caseFileArr: [],
        walkTime: moment().format('YYYY-MM-DD')
      },
      actions: [
        {text: '案件处置', message: '案件处置', icon: 'info-o', popupName: 'dispositionPopup'},
        {text: '案件上报', message: '案件上报', icon: 'comment-circle-o', popupName: 'reportingPopup'},
        {text: '案件暂存', message: '案件暂存', icon: 'more-o', popupName: 'cachePopup'}
      ],
      userSelection: {}, // 用户选择
      dataLoading: false,
      loading: false,
      caseTypeOpt: dictUtils.getDictItemsFromCache('tybagl-ajlx') || [], // 案件类型
      caseSrcOpt: dictUtils.getDictItemsFromCache('tybagl-ajl') || [], // 案件来源
      simpleHandleTypeOpt: dictUtils.getDictItemsFromCache('tybagl-czlx') || [], // 处置方式
      showPopover: false, // 多按钮弹层层显隐
      caseTypePicker: false, // 案件类型选择框显隐
      caseSrcPicker: false, // 案件来源选择框显隐
      walkTimeCalendar: false, // 巡逻时间日历选择
      dispositionPopup: false, // 案件处置处理表单弹出层显隐
      reportingPopup: false, //  案件上报部门表单弹出层显隐
      sectionTreeData: {} // 所选部门数据保存
    }
  },
  computed: {
    routerQuery() {
      return this.$route.query
    },
    previewUrl() {
      return window.context.portal + '/file/onlinePreviewController/v1/getFileById_'
    }
  },
  created() {
    if (this.routerQuery.id) {
      this.getDetails(this.routerQuery.id)
    }
  },
  methods: {
    moment,

    // 获取详情
    async getDetails(id) {
      this.dataLoading = true
      let formData = (await details(id)).value
      // 字段和字典对应
      const dictionaryOpt = {
        caseType: 'caseTypeOpt',
        caseSrc: 'caseSrcOpt',
        simpleHandleType: 'simpleHandleTypeOpt'
      }
      Object.keys(formData).forEach(key => {
        if (!dictionaryOpt[key]) return
        // 翻译对应的字段值
        const dictionary = this[dictionaryOpt[key]] // 字典
        const value = formData[key] // 表单值
        formData[key] = dictionaryEscape(dictionary, value, {
          congruent: false
        })
      })
      const {caseFile = ''} = formData
      this.formData = {
        ...formData,
        caseFile: caseFile ? caseFile?.split(',') : [],
        caseFileArr: caseFile ? caseFile?.split(',').map(id => ({
          url: this.previewUrl + id,
          isImage: true
        })) : []
      }
      this.dataLoading = false
    },

    /**
     * 重置表单
     * @param {Boolean} isConfirm 是否显示确认提示框
     * @returns void
     * */
    reset(isConfirm = false) {
      // 重置表单函数
      const resetForm = _ => {
        this.formData = this.$options.data().formData
        this.$refs.vanForm.resetValidation()
      }

      if (!isConfirm) return resetForm()

      Dialog.confirm({
        title: '',
        message: '确认重置录入信息？'
      }).then(_ => resetForm())
        .catch(_ => _)
    },

    // 多功能按钮弹层出现
    popoverOpen() {
      // loading状态下不允许出现功能按钮
      if (this.loading) return this.showPopover = false
    },

    // 多功能按钮点击
    popoverSelect(action) {
      const {popupName} = action
      this.userSelection = action
      switch (popupName) {
        // 案件处置处理
        case 'dispositionPopup':
          this.$refs.vanForm.submit()
          break
        // 案件上报部门
        case 'reportingPopup':
          this.$refs.vanForm.submit()
          break
        // 案件暂存缓存
        case 'cachePopup':
          this.submitForm({
            ...this.formData,
            caseStatus: '0'
          })
          break
      }
    },

    // 文件读取前
    beforeRead(file) {
      const isJPG =
        file.type === 'image/jpeg' ||
        file.type === 'image/png' ||
        file.type === 'image/jpg'
      const isLt1M = file.size / 1024 / 1024 < 10
      if (!isJPG) {
        Toast.fail('上传图片只能是 JPG 格式!')
      }
      if (!isLt1M) {
        Toast.fail('上传图片大小不能超过 1MB!')
      }
      return isJPG && isLt1M
    },

    // 文件读取后
    async afterRead(file) {
      if (this.loading) {
        Toast.fail('上传文件中...请上传后重试')
        this.formData.caseFileArr.pop()
      }
      this.loading = true
      const {fileId, success} = await fileUpload({
        files: file.file,
        bizCode: 'Image'
      })
      if (success && fileId) {
        file.fileId = fileId
        this.formData.caseFile.push(fileId)
      } else {
        Toast.fail('上传失败')
        this.formData.caseFileArr.pop()
      }
      this.loading = false
    },

    //  文件删除后
    beforeDelete(file) {
      if (this.loading) {
        Toast.fail('上传文件中...请上传后重试')
        return false
      }
      const index = this.formData.caseFile.findIndex(
        fileId => fileId === file.fileId
      )
      this.formData.caseFile.splice(index, 1)
      return true
    },


    //接收所属部门子组件
    handleOrg(val) {
      this.sectionTreeData = val
    },

    // 巡逻时间选择后
    walkTimeCalendarConfirm(time) {
      this.formData.walkTime = moment(time).format('YYYY-MM-DD')
      this.walkTimeCalendar = false
    },

    //所属部门确认
    onCancel() {
      const {name: handleDeptName = '', id: handleDeptId = ''} = this.sectionTreeData
      this.submitForm({
        ...this.formData,
        caseStatus: '1',
        handleType: 2,
        handleDeptName,
        handleDeptId
      })
    },

    // 表单验证成功
    onSubmit() {
      const {popupName} = this.userSelection
      this[popupName] = true

      switch (popupName) {
        // 案件处置处理
        case 'dispositionPopup':
          break
        // 案件上报部门
        case 'reportingPopup':
          //打开第二人员所属部门
          this.$nextTick(() => this.$refs.secondSectionTree.open())
          break
        // 案件暂存缓存
        case 'cachePopup':
          break
      }
    },

    // 表单验证失败
    onFailed({errors}) {
      if (errors.length) {
        Toast.fail(`有${errors.length}个字段未通过校验，请正确填写表单内容。`)
      }
    },

    // 弹层组件中表单提交验证成功
    popupFormSubmit(formData) {
      this.submitForm({
        ...this.formData,
        ...formData
      })
    },

    // 表单上传
    async submitForm(formData = {}) {
      this.loading = true
      // 字段和字典对应
      const dictionaryOpt = {
        caseType: 'caseTypeOpt',
        caseSrc: 'caseSrcOpt',
        simpleHandleType: 'simpleHandleTypeOpt'
      }
      let cloneFormData = _.cloneDeep(formData) || {}
      // 翻译对应的字段值
      Object.keys(cloneFormData).forEach(key => {
        if (!dictionaryOpt[key]) return
        const dictionary = this[dictionaryOpt[key]] // 字典
        const value = cloneFormData[key] // 表单值
        cloneFormData[key] = dictionaryEscape(dictionary, value, {
          dictionaryType: false,
          congruent: false
        })
      })
      const {caseFile = [], handleFile = []} = formData
      const {state} = await edit({
        ...cloneFormData,
        caseFile: caseFile?.join(',') || '',
        handleFile: handleFile?.join(',') || ''
      })
      const {message, popupName} = this.userSelection
      if (state) {
        Toast.success(`${message}成功`)
        this.formData = this.$options.data().formData
        const {cancel = _ => _} = this.$refs[popupName] || {}
        cancel() // 组件重置函数
        this.$router.push({
          path: 'caseHandling',
          query: {
            type: 'edit'
          }
        })
      } else {
        Toast.fail(`${message}失败，请重试`)
      }
      this.loading = false
    }
  }
}
</script>

<style lang="scss" scoped>
.form-container {
  width: 100%;
  height: 100%;
  box-sizing: border-box;
  display: flex;
  flex-direction: column;
  background: #fff;

  ::v-deep .van-form {
    flex: 1;
    overflow: auto;
    margin-bottom: 10vh;

    .footer {
      width: 100%;
      display: flex;
      align-items: center;
      height: 10vh;
      position: fixed;
      bottom: 0;
      justify-content: center;
      background: #fff;
      box-sizing: border-box;

      > .van-button {
        width: 40%;
        border-radius: 100px 0 0 100px;
        border-color: #999;
        border-width: 8px;
        border-right: 0;
      }

      .van-popover__wrapper {
        width: 40%;

        .van-button {
          width: 100%;
          border-radius: 0 100px 100px 0;
        }
      }
    }
  }

}

.placeholder {
  color: #c7c5c5;
}

::v-deep .van-action-sheet__close {
  display: none;
}

::v-deep .van-overlay {
  display: flex;
  justify-content: center;
  align-items: center;
}
</style>

<style lang="scss">
.van-popover__action {
  width: 110PX;
}
</style>
